module Vara
  class GitInspector
    # @param repo_path [String] Path to the root of the repository to inspect
    # @return [String] The SHA of the current git revision
    def self.current_revision(repo_path)
      new(repo_path).current_revision
    end

    # @param repo_path [String] Path to the root of the repository to inspect
    # @return [Boolean]
    def self.git_repo?(repo_path)
      system("cd #{repo_path} && git rev-parse --is-inside-work-tree")
    end

    # @param repo_path [String] Path to the root of the repository to inspect
    def initialize(repo_path)
      raise "The directory #{repo_path} must be a git repo" unless self.class.git_repo?(repo_path)
      @repo_path = repo_path
    end

    # @return [String] The SHA of the current git revision
    def current_revision
      revision = `cd #{repo_path} && git rev-list --max-count=1 HEAD`.chomp
      [revision, dirty_suffix].join('')
    end

    # @return [Integer] The number of commits on this git branch
    def commit_count
      count = `cd #{repo_path} && git rev-list HEAD | wc -l`
      Integer(count)
    end

    # @return [String] The abbreviated SHA of the current git revision
    def short_sha
      sha = `cd #{repo_path} && git rev-parse --short HEAD`.chomp
      [sha, dirty_suffix].join('')
    end

    private

    attr_reader :repo_path

    def dirty?
      # inspired by http://stackoverflow.com/a/5737794
      `cd #{repo_path} && test -n "$(git status --porcelain)"`
      $?.success?
    end

    def dirty_suffix
      if dirty?
        '.dirty'
      else
        ''
      end
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
