require 'yaml'

module Vara
  class Linter
    def self.build(metadata_path)
      new(YAML.load_file(metadata_path))
    end

    def initialize(metadata)
      @metadata = metadata
      @errors = []
    end

    def lint!
      validate_metadata_version
      raise LintError, errors.join("\n") unless errors.empty?
    end

    private

    def validate_metadata_version
      metadata_version = metadata.fetch('metadata_version')
      errors << 'metadata_version must be a string' unless metadata_version.is_a?(String)
    end

    attr_reader :metadata, :errors
  end

  class LintError < RuntimeError
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
