require 'digest/md5'

module Vara
  class ProductArtifactValidator
    def self.validate_file_checksum(file_metadata, file_name)
      raise "No checksum for file #{file_name}!" unless file_metadata.md5 || file_metadata.sha1

      validate_md5(file_metadata, file_name) if file_metadata.md5

      validate_sha1(file_metadata, file_name) if file_metadata.sha1
    end

    def self.validate_sha1(file_metadata, file_name)
      sha1_got = Digest::SHA1.file(file_name).hexdigest
      sha1_expected = file_metadata.sha1
      raise "sha1 mismatch for file #{file_name}, expected #{sha1_expected}, got #{sha1_got}" if sha1_got != sha1_expected
    end

    def self.validate_md5(file_metadata, file_name)
      md5_got = Digest::MD5.file(file_name).hexdigest
      md5_expected = file_metadata.md5
      raise "md5 mismatch for file #{file_name}, expected #{md5_expected}, got #{md5_got}" if md5_got != md5_expected
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
