require 'vara/product_metadata'
require 'vara/tarball'

module Vara
  class ProductResourceDownloader
    def self.build(product_metadata_path)
      product_metadata = Vara::ProductMetadata.from_file(product_metadata_path)
      product_basedir = File.expand_path(File.join(File.dirname(product_metadata_path), '..'))
      new(product_metadata, product_basedir)
    end

    attr_reader :product_basedir, :product_metadata

    def initialize(product_metadata, product_basedir)
      @product_basedir  = product_basedir
      @product_metadata = product_metadata
    end

    def download
      if product_metadata.explicit_stemcell? && product_metadata.stemcell_criteria?
        raise 'binaries.yml includes both stemcell and stemcell criteria keys'
      end

      if product_metadata.explicit_stemcell?
        download_stemcell
        download_compiled_packages
      end

      download_releases
    end

    private

    def download_stemcell
      stemcell_metadata   = product_metadata.stemcell_metadata
      stemcell_directory  = File.join(product_basedir, 'stemcells')
      stemcell = Vara::Tarball.new(stemcell_metadata, stemcell_directory)
      stemcell.sync_local_copy
    end

    def download_releases
      releases_metadata = product_metadata.releases_metadata

      releases_metadata.each do |release_metadata|
        release_directory = File.join(product_basedir, 'releases')
        release = Vara::Tarball.new(release_metadata, release_directory)
        release.sync_local_copy
      end
    end

    def download_compiled_packages
      return unless product_metadata.has_compiled_packages?

      compiled_packages_metadata   = product_metadata.compiled_packages_metadata
      compiled_packages_dir        = File.join(product_basedir, 'compiled_packages')
      compiled_package = Vara::Tarball.new(compiled_packages_metadata, compiled_packages_dir)
      compiled_package.sync_local_copy
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
